{
************************************************************
*   Phase Detector version 1.0                             *
*                                                          *
*   DEMO - Detect Phase difference between two input pins  *
*         (C) 2007 Parallax, Inc.                          *
************************************************************

    coded by Beau Schwabe (Parallax)

See 'PhaseDetector.spin' for theory of operation.
    
}                           
CON
{{
                                TV Setup Constants  
}}
  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000
  _stack = ($3000 + $3000 + 100) >> 2                   'accommodate display memory and stack

  x_tiles = 16
  y_tiles = 12

  paramcount = 14       
  bitmap_base = $2000
  display_base = $5000

VAR
{{
                                TV Setup Variables  
}}
  long  tv_status     '0/1/2 = off/visible/invisible           read-only
  long  tv_enable     '0/? = off/on                            write-only
  long  tv_pins       '%ppmmm = pins                           write-only
  long  tv_mode       '%ccinp = chroma,interlace,ntsc/pal,swap write-only
  long  tv_screen     'pointer to screen (words)               write-only
  long  tv_colors     'pointer to colors (longs)               write-only               
  long  tv_hc         'horizontal cells                        write-only
  long  tv_vc         'vertical cells                          write-only
  long  tv_hx         'horizontal cell expansion               write-only
  long  tv_vx         'vertical cell expansion                 write-only
  long  tv_ho         'horizontal offset                       write-only
  long  tv_vo         'vertical offset                         write-only
  long  tv_broadcast  'broadcast frequency (Hz)                write-only
  long  tv_auralcog   'aural fm cog                            write-only

  word  screen[x_tiles * y_tiles]
  long  colors[64]

OBJ

  tv    : "tv"
  gr    : "graphics"
  Phase : "PhaseDetector"
  Freq  : "Synth"

CON
{{
                         Phase Detector Demo Constants  
}}
   PinA = 0                     'Input Pin1
   PinB = 1                     'Input Pin2

   PinC = 7                     'Output Pin             DEMO Only
   FreqTest = 1_000_000         'Output Frequency       DEMO Only
{{

For a loop-back test (This DEMO) connect PinA, PinB, and PinC all together

}}   
   
PUB start | i, dx, dy

  'start tv
  longmove(@tv_status, @tvparams, paramcount)
  tv_screen := @screen
  tv_colors := @colors
  tv.start(@tv_status)

  'init colors
  repeat i from 0 to 63
    colors[i] := $2B069C92

  'init tile screen
  repeat dx from 0 to tv_hc - 1
    repeat dy from 0 to tv_vc - 1
      screen[dy * tv_hc + dx] := display_base >> 6 + dy + dx * tv_vc + ((dy & $3F) << 10)

  'start and setup graphics
  gr.start
  gr.setup(16, 12, 128, 96, bitmap_base)

  Phase.start(PinA, PinB)         'Uses CTRA
  Freq.Synth("B", PinC, FreqTest) 'Use  CTRB since CTRA is already used by the Phase Detector

  gr.colorwidth(2, 0)                                   'Set Color and Width
  gr.textmode(1,1,7,%0000)                              'Set text mode

  repeat

    gr.clear                                            'Clear graphics screen

    gr.text(-100,80,string("Phase Detector Demo"))      'Display Header Text

    gr.text(-100,60,string("Phase ="))                  'Display Phase Value                    
    SimpleNum(0,60,Phase.Value,0)

    gr.copy(display_base)                               'copy bitmap to display

PUB SimpleNum(x,y,DecimalNumber,DecimalPoint)|sign,DecimalIndex,TempNum,spacing,DecimalFlag,Digit,Zero,DP,Hyphen
{     x,y           - lower right text coordinate
      DecimalNumber - signed Decimal number
      DecimalPoint  - number of places from the Right the decimal point should be
}
    Zero := $30
    DP := $2E
    Hyphen := $2D 

    spacing := 7
    DecimalIndex := 0

    TempNum := DecimalNumber                            'Preserve sign of DecimalNumber
    DecimalNumber := ||DecimalNumber
    if DecimalNumber <> TempNum 
       sign := 1
    else
       sign := 0

    repeat                                              'Print digits
      if DecimalIndex == DecimalPoint
         gr.text(x,y,@DP)                               'Insert decimal point at proper location
         x := x - spacing
         

      TempNum := DecimalNumber                          'Extract the least significant digit
      TempNum := DecimalNumber - ((TempNum / 10) * 10)

      Digit := $30 + TempNum                            'Display the least significant digit
      gr.text(x,y,@Digit)

      x := x - spacing
      DecimalIndex := DecimalIndex + 1
      DecimalNumber := DecimalNumber / 10               'Divide DecimalNumber by 10 

      if DecimalNumber == 0                             'Exit logic
         repeat while DecimalIndex < DecimalPoint       '   Do this if DecimalNumber is less than where the decimal point should be
            gr.text(x,y,@Zero)
            x := x - spacing
            DecimalIndex := DecimalIndex + 1
            DecimalFlag := 1
         if DecimalIndex == DecimalPoint                '   Set flag if DecimalNumber is equal to where the decimal point should be  
            DecimalFlag := 1   
         if DecimalFlag == 1
            gr.text(x,y,@DP)                            '   Insert decimal and leading Zero
            x := x - spacing
            gr.text(x,y,@Zero)                          
            x := x - spacing                
         if sign == 1                                   '   Restore sign of DecimalNumber
            gr.text(x,y,@Hyphen)
         quit

DAT
{{
                                TV Setup Varaibles  
}}
tvparams                long    0               'status
                        long    1               'enable
                        long    %001_0101       'pins
                        long    %0000           'mode
                        long    0               'screen
                        long    0               'colors
                        long    x_tiles         'hc
                        long    y_tiles         'vc
                        long    10              'hx
                        long    1               'vx
                        long    0               'ho
                        long    0               'vo
                        long    0               'broadcast
                        long    0               'auralcog